from __future__ import unicode_literals

import logging

from mopidy.backends import base
from mopidy.models import SearchResult

logger = logging.getLogger('mopidy.backends.beets')


class BeetsLibraryProvider(base.BaseLibraryProvider):
    def __init__(self, *args, **kwargs):
        super(BeetsLibraryProvider, self).__init__(*args, **kwargs)
        self.remote = self.backend.beets_api

    def find_exact(self, query=None, uris=None):
            return self.search(query=query, uris=uris)

    def search(self, query=None, uris=None):
        if not self.remote.has_connection:
            return []
            
        if not query:
            # Fetch all data(browse library)
            return SearchResult(
                uri='beets:search',
                tracks=self.remote.get_tracks())

        self._validate_query(query)

        for (field, val) in query.iteritems():
            if field == "album":
                return SearchResult(
                    uri='beets:search',
                    tracks=self.remote.get_album_by(val[0]) or [])
            elif field == "artist":
                return SearchResult(
                    uri='beets:search',
                    tracks=self.remote.get_item_by(val[0]) or [])
            elif field == "any":
                return SearchResult(
                    uri='beets:search',
                    tracks=self.remote.get_item_by(val[0]) or [])
            else:
                raise LookupError('Invalid lookup field: %s' % field)

        return []

    def lookup(self, uri):
        try:
            id = uri.split(";")[1]
            logger.debug(u'Beets track id for "%s": %s', id, uri)
            return [self.remote.get_track(id, True)]
        except Exception as error:
            logger.debug(u'Failed to lookup "%s": %s', uri, error)
            return []

    def _validate_query(self, query):
        for (_, values) in query.iteritems():
            if not values:
                raise LookupError('Missing query')
            for value in values:
                if not value:
                    raise LookupError('Missing query')



from __future__ import unicode_literals

import os
from mopidy import ext, config
from mopidy.exceptions import ExtensionError


__doc__ = """A extension for playing music from Beets.

This extension handles URIs starting with ``beets:`` and enables you,
to play music from Beets web service.

See https://github.com/dz0ny/mopidy-beets/ for further instructions on
using this extension.

**Issues:**

https://github.com/dz0ny/mopidy-beets/issues

**Dependencies:**

requests

"""

__version__ = '1.0.1'


class BeetsExtension(ext.Extension):

    dist_name = 'Mopidy-Beets'
    ext_name = 'beets'
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(BeetsExtension, self).get_config_schema()
        schema['hostname'] = config.Hostname()
        schema['port'] = config.Port()
        return schema

    def validate_environment(self):
        try:
            import requests  # noqa
        except ImportError as e:
            raise ExtensionError('Library requests not found', e)

    def get_backend_classes(self):
        from .actor import BeetsBackend
        return [BeetsBackend]
